/** @mainpage Doxygen Example Program
*
* This is a example program to demonstrate the documentation tool Doxygen.<br>See the corresponding Doxygen tutorial on <a href="http://www.cypax.net">www.cypax.net</a>
* <br>This program will sort arrays of integer by using different sorting algorithms.
*
* <img src="application_screenshot.jpg" alt="Screenshot">
*
* @author Philipp Bank
**/


/**
* @file DoxygenExample.java
*
* @brief This file contains the DoxygenExample class with the main() function.
*
* @author Philipp Bank, www.cypax.net\n
*
* @date June, 18th 2006 - first version
* @date June, 19th 2006 - some modifications, multi-threading
*
**/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import sorting.*;


/**
* @class DoxygenExample
* 
* @brief The DoxygenExample class extends the Frame class, provides a graphical user interface (GUI) and includes main().
*
* This class, extending the Frame class, contains the main() function, which will create a new DoxygenExample frame instance.<br>
* The program will sort integer arrays with:
* <ul> 
*   <li> a <a href="http://en.wikipedia.org/wiki/Selection_sort">selectionsort</a> algorithm</li>
*   <li> a <a href="http://en.wikipedia.org/wiki/Insertion_sort">insertionsort</a> algorithm</li>
*   <li> a <a href="http://en.wikipedia.org/wiki/Bubblesort">bubblesort</a> algorithm</li>
* </ul>
* The algorithms are executed as threads and can be started simultaneously or separately.<br>
* To visualize the processes, the integer arrays will be shown as colored lines.
*/
public class DoxygenExample extends Frame
{

  //three panels, used for the visualisation of the algorithm processes
  private Panel ssPanel = new Panel(null);
  private Panel isPanel = new Panel(null);
  private Panel bsPanel = new Panel(null);
  
  //three labels for the algorithm names
  private Label label1 = new Label();
  private Label label2 = new Label();
  private Label label3 = new Label();
  
  //a panel, containig the buttons
  private Panel buttonPanel = new Panel(null);
  
  private Button startButton = new Button();
  private Button randomizeButton = new Button();
  private Button randStartButton = new Button();
  private Button ssButton = new Button();
  private Button isButton = new Button();
  private Button bsButton = new Button();
  private Button exitButton = new Button();

  //the TimeLabel are used to show the runtime of the algorithms
  private Label ssTimeLabel = new Label();
  private Label isTimeLabel = new Label();
  private Label bsTimeLabel = new Label();

  private int[] ssArray = new int[210];    // the Array for the selectionsort algorithm
  private int[] isArray = new int[210];    //           for the insertionsort
  private int[] bsArray = new int[210];    //           for the bubblesort


/**
*
* @brief The constructor generates and places the graphical componets of the DoxygenExample frame.
*
* @param title - The title for the frame.
*
*/
  public DoxygenExample(String title) 
  {
    super(title);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) { System.exit(0); }
    });
    
    int frameWidth = 900;
    int frameHeight = 400;
    setSize(frameWidth, frameHeight);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int x = (d.width - getSize().width) / 2;
    int y = (d.height - getSize().height) / 2 ;
    setLocation(x, y);
    Panel cp = new Panel(null);
    add(cp);

    this.setBackground(new Color(240, 240, 240));  

    ssPanel.setBounds(16, 40, 250, 200);
    cp.add(ssPanel);
    isPanel.setBounds(324, 40, 250, 200);
    cp.add(isPanel);
    bsPanel.setBounds(632, 40, 250, 200);
    cp.add(bsPanel);
    label1.setBounds(16, 16, 90, 16);
    label1.setText("Selectionsort");
    label1.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(label1);
    label2.setBounds(324, 16, 90, 16);
    label2.setText("Insertionsort");
    label2.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(label2);
    label3.setBounds(632, 16, 90, 16);
    label3.setText("Bubblesort");
    label3.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(label3);
    buttonPanel.setBounds(16, 280, 860, 81);
    cp.add(buttonPanel);
    startButton.setBounds(140, 48, 110, 25);
    randomizeButton.setBounds(16, 48, 110, 25);
    randStartButton.setBounds(380, 48, 110, 25);
    ssButton.setBounds(16,5,230,25);
    ssButton.setLabel("start");
    startButton.setLabel("start sorting");
    randomizeButton.setLabel("ramdomize");
    randStartButton.setLabel("randomized start");
    buttonPanel.add(randStartButton);
    buttonPanel.add(startButton);
    buttonPanel.add(randomizeButton);
    buttonPanel.add(ssButton);
    buttonPanel.add(isButton);
    buttonPanel.add(bsButton);

    ssButton.setBounds(16,10,220,25);
    ssButton.setLabel("start selectionsort");
    isButton.setBounds(324,10,220,25);
    isButton.setLabel("start insertionsort");
    bsButton.setBounds(632,10,220,25);
    bsButton.setLabel("start bubblesort");

    ssPanel.setBackground(new Color(128,128,128));
    isPanel.setBackground(new Color(128,128,128));
    bsPanel.setBackground(new Color(128,128,128));
    buttonPanel.setBackground(new Color(192,192,192));

    
    randomizeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        randomizeButtonActionPerformed(evt);
      }
    });

    randStartButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        randStartButtonActionPerformed(evt);
      }
    });

    startButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        startButtonActionPerformed(evt);
      }
    });

    ssButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        ssButtonActionPerformed(evt);
      }
    });

    isButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        isButtonActionPerformed(evt);
      }
    });

    bsButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        bsButtonActionPerformed(evt);
      }
    });


    exitButton.setBounds(772, 48, 75, 25);
    exitButton.setLabel("exit");
    buttonPanel.add(exitButton);

    exitButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        exitButtonActionPerformed(evt);
      }
    });

    ssTimeLabel.setBounds(16, 256, 120, 16);
    ssTimeLabel.setText("  ");
    ssTimeLabel.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(ssTimeLabel);
    isTimeLabel.setBounds(324, 256, 120, 16);
    isTimeLabel.setText("  ");
    isTimeLabel.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(isTimeLabel);
    bsTimeLabel.setBounds(632, 256, 120, 16);
    bsTimeLabel.setText("  ");
    bsTimeLabel.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
    cp.add(bsTimeLabel);

    setResizable(false);
    setVisible(true);
    
    this.randomizeArrays();  //ramdomize the arrays
  }


/**
* @brief This function fills the arrays with randomly generated numbers and then calls the showArray() function to visualize the new numbers.
*/
  public void randomizeArrays()
  {
    
    //generate new random numbers
    Random r = new Random(System.currentTimeMillis()); 
   
    for (int i=0; i<210; i++)
    {
      ssArray[i] = 1 + (Math.abs(r.nextInt()) % 180);
      //same numbers in all arrays
      isArray[i] = ssArray[i];
      bsArray[i] = ssArray[i];
    }

    //visualize the generated numbers
    Paint.showArray(ssArray, ssPanel);
    Paint.showArray(isArray, isPanel);
    Paint.showArray(bsArray, bsPanel);
  }

/**
* @brief This function generates and starts a new Thread for the selectionsort algorithm.
*/
  public void startSelectionSort()
  {
    SelectionSort sSort = new SelectionSort();
    sSort.setSettings(this.ssArray, this.ssPanel, this.ssTimeLabel);
    Thread ssThread = new Thread(sSort);  //generate new thread

    ssThread.start();
    
    //wait until thread has finished
    try
    {
      ssThread.join();
    }
    catch (InterruptedException ie)
    {

    }
  }

/**
* @brief This function generates and starts a new Thread for the insertionsort algorithm.
*/
  public void startInsertionSort()
  {
    InsertionSort iSort = new InsertionSort();
    iSort.setSettings(this.isArray, this.isPanel, this.isTimeLabel);
    Thread isThread = new Thread(iSort);   //generate new thread


    isThread.start();

    //wait until thread has finished
    try
    {
      isThread.join();
    }
    catch (InterruptedException ie)
    {

    }
  }

/**
* @brief This function generates and starts a new Thread for the bubblensort algorithm.
*/
  public void startBubbleSort()
  {
    BubbleSort bSort = new BubbleSort();
    bSort.setSettings(this.bsArray, this.bsPanel, this.bsTimeLabel);
    Thread bsThread = new Thread(bSort);  //generate new thread


    bsThread.start();

    //wait until thread has finished
    try
    {
      bsThread.join();
    }
    catch (InterruptedException ie)
    {

    }
  }

/**
* @brief This function generates and starts all sorting algorithms simultaneously.
*/
  public void sortArrays()
  {
    SelectionSort sSort = new SelectionSort();
    sSort.setSettings(this.ssArray, this.ssPanel, this.ssTimeLabel);
    Thread ssThread = new Thread(sSort);  //generate new thread

    InsertionSort iSort = new InsertionSort();
    iSort.setSettings(this.isArray, this.isPanel, this.isTimeLabel);
    Thread isThread = new Thread(iSort);  //generate new thread

    BubbleSort bSort = new BubbleSort();
    bSort.setSettings(this.bsArray, this.bsPanel, this.bsTimeLabel);
    Thread bsThread = new Thread(bSort);  //generate new thread


    /* Ok, this is not realy a simultaneously start - as you can see below, the starting order is: selectionsort, insertionsort and then bubblesort.
       But as soon as one thread has been started via start(),
       the next thread will be started and doesn't need to wait until the previous thread has finished. */
    ssThread.start();
    isThread.start();
    bsThread.start();


    //wait until threads are finished
    try
    {
      ssThread.join();
    }
    catch (InterruptedException ie)
    {

    }

    try
    {
      isThread.join();
    }
    catch (InterruptedException ie)
    {

    }

    try
    {
      bsThread.join();
    }
    catch (InterruptedException ie)
    {

    }
  }

/**
* @brief This is the action handling for the ssButton.
* This function will call the startSelectionSort() function to start a selectionsort thread.
*/
  public void ssButtonActionPerformed(ActionEvent evt)
  {
    this.startSelectionSort();
  }

/**
* @brief This is the action handling for the isButton.
* This function will call the startInsertionSort() function to start a insertionsort thread.
*/
  public void isButtonActionPerformed(ActionEvent evt)
  {
    this.startInsertionSort();
  }

/**
* @brief This is the action handling for the bsButton.
* This function will call the startBubbleSort() function to start a bubblesort thread.
*/
  public void bsButtonActionPerformed(ActionEvent evt)
  {
    this.startBubbleSort();
  }

/**
* @brief This is the action handling for the randomizeButton.
* By pressing this button this function will call the randomizeArrays() function, which fills the arrays with new random numbers.
*/
  public void randomizeButtonActionPerformed(ActionEvent evt)
  {
    this.randomizeArrays();
  }

/**
* @brief This is the action handling for the randStartButton.
* This function will do exactly the same tasks as randomizeButtonActionPerformed(), but will also call the sortArrays() function
* to sort the arrays with simultaneously running sorting threads.
*/
  public void randStartButtonActionPerformed(ActionEvent evt)
  {
    this.randomizeArrays();
    this.sortArrays();
  }

/**
* @brief This is the action handling for the startButton.
* By pressing this button, the sortArrays() function will be called without(!) randomize the arrays.
* Note, that most algorithms run much faster if the array is aready sorted.
*/
  public void startButtonActionPerformed(ActionEvent evt)
  {
    this.sortArrays();
  }

/**
* @brief This is the action handling for the exitButton.
* By pressing this button, the application will terminate.
*/
  public void exitButtonActionPerformed(ActionEvent evt)
  {
    System.exit(0);
  }

/**
* @brief The main function.
* The one and only job for this function is to generate a new DoxygenExample instance.
*/
  public static void main(String[] args)
  {
    new DoxygenExample("Selectionsort, Insertionsort and Bubblesort");
  }
}

